﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.IO;

namespace Persistence.Test {

  public class Root {
    [Storable]
    public int[] i = new int[] { 3, 4, 5, 6 };
    [Storable]
    public string s;
    [Storable]
    public ArrayList intArray = new ArrayList(new int[] { 1, 2, 3 });
    [Storable]
    public List<int> intList = new List<int>(new int[] { 321, 312, 321 });
    [Storable]
    public Custom c;
    [Storable]
    public List<Root> selfReferences;
    [Storable]
    public double[,] multiDimArray = new double[,] { { 1, 2, 3 }, { 3, 4, 5 } };
    [Storable]
    public bool boolean = true;
    [Storable]
    public DateTime dateTime = new DateTime();
    [Storable]
    public KeyValuePair<string, int> kvp = new KeyValuePair<string, int>("test key", 123);
    [Storable]
    public Dictionary<string, int> dict = new Dictionary<string, int>();
  }

  public class Custom {
    [Storable]
    public int i;
    [Storable]
    public Root r;
    [Storable]
    public string name = "Serial";
  }

  public class CloneableRoot {
    public int[] i = new int[] { 3, 4, 5, 6 };
    public string s;
    public ArrayList intArray = new ArrayList(new int[] { 1, 2, 3 });
    public List<int> intList = new List<int>(new int[] { 321, 312, 321 });
    public CloneableCustom c;
    public List<CloneableRoot> selfReferences;
    public double[,] multiDimArray = new double[,] { { 1, 2, 3 }, { 3, 4, 5 } };
    public bool boolean = true;
    public DateTime dateTime = new DateTime();
    public KeyValuePair<string, int> kvp = new KeyValuePair<string, int>("test key", 123);
    public Dictionary<string, int> dict = new Dictionary<string, int>();    
    public object Clone(Dictionary<object, object> twins) {
      if (twins.ContainsKey(this))
        return twins[this];      
      CloneableRoot cr = new CloneableRoot();
      twins.Add(this, cr);
      cr.i = this.i;
      cr.s = this.s;
      cr.intArray = new ArrayList(this.intArray);
      cr.intList = new List<int>(this.intList);
      cr.c = (CloneableCustom)c.Clone(twins);
      cr.selfReferences = new List<CloneableRoot>();
      for (int i = 0; i < this.selfReferences.Count; i++) {
        cr.selfReferences.Add(this);
      }
      cr.multiDimArray = (double[,])this.multiDimArray.Clone();
      cr.dateTime = new DateTime(this.dateTime.Ticks);
      cr.kvp = new KeyValuePair<string, int>(this.kvp.Key, this.kvp.Value);
      cr.dict = new Dictionary<string, int>(this.dict);
      return cr;
    }    
  }

  public class CloneableCustom {    
    public int i;    
    public CloneableRoot r;    
    public string name = "Serial";
    public object Clone(Dictionary<object, object> twins) {
      if (twins.ContainsKey(this))
        return twins[this];
      CloneableCustom cc = new CloneableCustom();
      twins.Add(this, cc);
      cc.i = this.i;
      cc.r = (CloneableRoot)this.r.Clone(twins);
      cc.name = this.name;
      return cc;
    }
  }

  public class Manager {

    private DateTime lastLoadTime;
    [Storable]
    private DateTime lastLoadTimePersistence {
      get { return lastLoadTime; }
      set { this.lastLoadTime = DateTime.Now; }
    }
    [Storable]
    private Nullable<double> dbl = null;
  }

  public class StorableObject {

    [Storable]
    Dictionary<int, string> dict;

    public void Init() {
      this.dict = new Dictionary<int, string>();
      for (int i = 0; i < 1000000; i++) {
        dict.Add(i, i.ToString());
      }
    }
  }

  public class CloneableObject : ICloneable {

    Dictionary<int, string> dict;

    public void Init() {
      this.dict = new Dictionary<int, string>();
      for (int i = 0; i < 1000000; i++) {
        dict.Add(i, i.ToString());
      }
    }
    public object Clone() {
      CloneableObject clone = new CloneableObject();
      clone.dict = new Dictionary<int, string>(this.dict);
      return clone;
    }
  }  

  public class NewSerializationTest {

    public static void Test1() {
      Root r = new Root();
      r.selfReferences = new List<Root>();
      r.selfReferences.Add(r);
      r.selfReferences.Add(r);
      r.c = new Custom();
      r.c.r = r;
      r.dict.Add("one", 1);
      r.dict.Add("two", 2);
      r.dict.Add("three", 3);
      Serializer s = new Serializer(r);        
      Persistence.XmlFormatter xmlFormatter = new Persistence.XmlFormatter();
      StreamWriter writer = new StreamWriter("test.xml");
      foreach (ISerializationToken token in s) {
        string line = xmlFormatter.Format(token);
        writer.Write(line);
        Console.Out.Write(line);
      }
      writer.Close();
      XmlParser parser = new XmlParser(new StreamReader("test.xml"));
      DeSerializer deSerializer = new DeSerializer();        
      object o = deSerializer.DeSerialize(parser);
      Root t = CloningFactory.DefaultClone(r);
      Console.Out.WriteLine(Util.AutoFormat(o, true));      
    }

    public static void Test2() {
      Manager m = new Manager();      
      Serializer s = new Serializer(m);
      Persistence.XmlFormatter xmlFormatter = new Persistence.XmlFormatter();
      StreamWriter writer = new StreamWriter("test2.xml");
      foreach (ISerializationToken token in s) {
        string line = xmlFormatter.Format(token);
        writer.Write(line);
        Console.Out.Write(line);
      }
      writer.Close();
      XmlParser parser = new XmlParser(new StreamReader("test2.xml"));
      DeSerializer deSerializer = new DeSerializer();
      object o = deSerializer.DeSerialize(parser);
      Manager n = CloningFactory.DefaultClone(m);
      Console.Out.WriteLine(Util.AutoFormat(o, true));      
    }

    public static void SpeedTest() {
      StorableObject storable = new StorableObject();
      CloneableObject cloneable = new CloneableObject();
      Console.Write("initializing...");
      storable.Init();
      cloneable.Init();
      Console.WriteLine("done");
      List<StorableObject> storableClones = new List<StorableObject>();
      List<CloneableObject> clonableClones = new List<CloneableObject>();
      CloningFactory cloningFactory = new CloningFactory();
      for (int i = 0; i < 100000; i++) {
        Console.Write("cloning storable.. ");
        storableClones.Add(cloningFactory.Clone(storable));
        Console.WriteLine();
        Console.Write("cloning cloneable.. ");
        clonableClones.Add((CloneableObject)cloneable.Clone());
        Console.WriteLine();
      }
    }

    public static void SpeedTest2() {
      Root r = new Root();
      r.selfReferences = new List<Root>();
      r.selfReferences.Add(r);
      r.selfReferences.Add(r);
      r.c = new Custom();
      r.c.r = r;
      r.dict.Add("one", 1);
      r.dict.Add("two", 2);
      r.dict.Add("three", 3);

      CloneableRoot cr = new CloneableRoot();
      cr.selfReferences = new List<CloneableRoot>();
      cr.selfReferences.Add(cr);
      cr.selfReferences.Add(cr);
      cr.c = new CloneableCustom();
      cr.c.r = cr;
      cr.dict.Add("one", 1);
      cr.dict.Add("two", 2);
      cr.dict.Add("three", 3);      
      List<Root> storableClones = new List<Root>();
      List<CloneableRoot> clonableClones = new List<CloneableRoot>();      
      CloningFactory cloningFactory = new CloningFactory();

      DateTime start = DateTime.Now;
      Console.Write("cloning storable.. ");
      for (int i = 0; i < 100000; i++) {       
        storableClones.Add(cloningFactory.Clone(r));                        
      }
      Console.WriteLine(new TimeSpan(DateTime.Now.Ticks - start.Ticks));

      start = DateTime.Now;
      Console.Write("cloning storable again.. ");
      for (int i = 0; i < 100000; i++) {
        storableClones.Add(cloningFactory.Clone(r));
      }
      Console.WriteLine(new TimeSpan(DateTime.Now.Ticks - start.Ticks));


      start = DateTime.Now;
      Console.Write("cloning cloneable.. ");
      for (int i = 0; i < 100000; i++) {        
        clonableClones.Add((CloneableRoot)cr.Clone(new Dictionary<object, object>()));
      }
      Console.WriteLine(new TimeSpan(DateTime.Now.Ticks - start.Ticks));
      clonableClones.Clear();
      
    }

    public static void Main() {
      //Test1();
      //Test2();
      //SpeedTest();
      SpeedTest2();
      Console.In.ReadLine();
    }
  }
}
